# Zope
from OFS import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from AccessControl import ClassSecurityInfo

# cmf
from Products.CMFCore.utils import getToolByName, UniqueObject
from Products.CMFCore.ActionProviderBase import ActionProviderBase


class Populator:
    
    """
    Populates a container with this content
    """
    
    def __init__(self, container):
        self.container = container
        self.items = []
        
    def addItem(self, item):
        self.items.append(item)

    def add(self, id, portal_type, fixed_values=None, **attrs):
        if fixed_values is None:
            fixed_values = []
        item = {'id':id, 'portal_type':portal_type, 'fixed_values':fixed_values, 'attrs':attrs}
        self.addItem(item)

    def addNew(self, item):
        container = self.container
        id = item['id']
        attrs = item['attrs']
        container.invokeFactory(item['portal_type'], id, **attrs)
        obj = getattr(container, id)
        obj.manage_fixupOwnershipAfterAdd()
        return obj

    def _accessor(self, obj, attr):
        return obj.schema[attr].getAccessor(obj)

    def _mutator(self, obj, attr):
        return obj.schema[attr].getMutator(obj)

    def getIds(self):
        return [item['id'] for item in self.items]

    def unreserve(self):
        container = self.container.aq_base
        if getattr(container, '_reserved_names', ()):
            del container._reserved_names

    def reserve(self):
        container = self.container
        reserved_names = tuple(self.getIds())
        if reserved_names and (getattr(container.aq_base, '_reserved_names', ()) != reserved_names):
            container._reserved_names = reserved_names

    def populate_item(self, item):
        container = self.container
        id = item['id']
        obj = getattr(container, id, None)
        if obj is None:
            self.unreserve()
            obj = self.addNew(item)
        else:
            # check if any locked attr has been changed.
            fixeds = item.get('fixed_attrs', [])
            attrs = item.get('attrs', [])
            mutated = 0
            for fixed in fixeds:
                accessor = self._accessor(obj, fixed)
                if accessor() != attrs[fixed]:
                    mutated = 1
                    mutator = self._mutator(obj, fixed)
                    mutator(attrs[fixed])
            if mutated:
                obj.reindexObject()
        # run config script if any
        if 'config_script' in item:
            config_script = getattr(obj, item['config_script'])
            config_script()
        # add subcontent
        if 'subitems' in item:
            populator = Populator(obj)
            for subitem in item['subitems']:
                populator.addItem(subitem)
            populator.populate()
        

    def populate(self):
        if not self.items:
            self.unreserve()
        for item in self.items:
            try:
                self.populate_item(item)
            except: # we don't want to break the site because of a bad config script
                pass
        self.reserve()
                



class PopulatorTool(UniqueObject, PropertyManager, 
                    SimpleItem.SimpleItem, ActionProviderBase):
    "The PopulatorTool"

    meta_type = 'PopulatorTool'

    id = 'populator_tool'

    manage_options = PropertyManager.manage_options + \
                     SimpleItem.SimpleItem.manage_options + (
    	{'label': 'View', 'action': 'index_html',},
    )

    _properties = (
        {'id':'title', 'type':'string', 'mode':'w'},
    )

    # Standard security settings
    security = ClassSecurityInfo()

    security.declareProtected('Manage properties', 'index_html')
    index_html = PageTemplateFile('zpt/index_html', globals())

    security.declarePublic('populate')
    def populate(self, obj, script_name):
        populator = Populator(obj)
        config_script = getattr(self, script_name)
        items = config_script()
        for item in items:
            populator.addItem(item)
        populator.populate()


    security.declarePublic('listPopulatedItems')
    def listPopulatedItems(self, obj, script_name):
        "Returns the populated objects from object, if they exist and authenticated has view permisson"
        member = getToolByName(self, 'portal_membership').getAuthenticatedMember()
        config_script = getattr(self, script_name)
        items = config_script()
        result = []
        for item in items:
            content = getattr(obj.aq_base, item['id'], None)
            if (not content is None) and member.has_permission('View', content):
                result.append(content)
        return result
            
        

#####################################################
# Constructor functions, only used when adding class
# to objectManager

def manage_addAction(self, REQUEST=None):
    "Add tool instance to parent ObjectManager"
    id = PopulatorTool.id
    self._setObject(id, PopulatorTool())
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

constructors = (manage_addAction,)

